/****** Imports vom Startup-File fr den STM32F103ZET6 auf dem Eigenbau-Eval-Board ******/


#ifndef STM32F103_INCLUDED
#define STM32F103_INCLUDED

/* memory mapping struct for System Control Block.. wo: ARM    DDI0337E_cortex_m3_r1p1_trm.pdf */
struct T_SCB
{ volatile unsigned long CPUID;     /* CPU ID Base Register                                     */
  volatile unsigned long ICSR;      /* Interrupt Control State Register                         */
  volatile unsigned long VTOR;      /* Vector Table Offset Register                             */
  volatile unsigned long AIRCR;     /* Application Interrupt / Reset Control Register           */
  volatile unsigned long SCR;       /* System Control Register                                  */
  volatile unsigned long CCR;       /* Configuration Control Register                           */
  volatile unsigned char SHP[12];   /* System Handlers Priority Registers (4-7, 8-11, 12-15)    */
  volatile unsigned long SHCSR;     /* System Handler Control and State Register                */
  volatile unsigned long CFSR;      /* Configurable Fault Status Register                       */
  volatile unsigned long HFSR;      /* Hard Fault Status Register                               */
  volatile unsigned long DFSR;      /* Debug Fault Status Register                              */
  volatile unsigned long MMFAR;     /* Mem Manage Address Register                              */
  volatile unsigned long BFAR;      /* Bus Fault Address Register                               */
  volatile unsigned long AFSR;      /* Auxiliary Fault Status Register                          */
  volatile unsigned long PFR[2];    /* Processor Feature Register                               */
  volatile unsigned long DFR;       /* Debug Feature Register                                   */
  volatile unsigned long ADR;       /* Auxiliary Feature Register                               */
  volatile unsigned long MMFR[4];   /* Memory Model Feature Register                            */
  volatile unsigned long ISAR[5];   /* ISA Feature Register                                     */
};

#define SCB   ((struct T_SCB*) (0xE000ED00))

/* NVIC  Nested  Interrupt Controller */
#define __NVIC_PRIO_BITS    4       /* standard definition for NVIC Priority Bits */

struct T_NVIC
{ volatile unsigned long ISER[8];                      /*!< Interrupt Set Enable Register            */
           unsigned long RESERVED0[24];
  volatile unsigned long ICER[8];                      /*!< Interrupt Clear Enable Register          */
           unsigned long RSERVED1[24];
  volatile unsigned long ISPR[8];                      /*!< Interrupt Set Pending Register           */
           unsigned long RESERVED2[24];
  volatile unsigned long ICPR[8];                      /*!< Interrupt Clear Pending Register         */
           unsigned long RESERVED3[24];
  volatile unsigned long IABR[8];                      /*!< Interrupt Active bit Register            */
           unsigned long RESERVED4[56];
  volatile unsigned char IP[240];                      /*!< Interrupt Priority Register, 8Bit wide   */
           unsigned long RESERVED5[644];
  volatile unsigned long STIR;                         /*!< Software Trigger Interrupt Register      */
};

#define NVIC   ((struct T_NVIC*) (0xE000E100))


/* Systick (in der Cortex-CPU eingebaut, ebenfalls ARM DDI0337E_cortex_m3_r1p1_trm.pdf */
/* Max Count = 1E24 -1,
   in SYSTICK_CTRL:
   Bit  0 = Enable,
   Bit  1 = Enable Interrupt,
   Bit  3 = Clocksource: Hi --> HCLK, Lo --> HCLK/8
   Bit 16 = wir dbei jedem Reload gesetzt, per Soft zurckzustellen
 */
#define SYSTICK_CTRL_ENABLE     1           /* Counter enable */
#define SYSTICK_CTRL_TICKINT    2           /* Counting down to 0 pends the SysTick handler */
#define SYSTICK_CTRL_CLKSOURCE  4           /* Clock source */
#define SYSTICK_CTRL_COUNTFLAG  0x00010000  /* Count Flag */

#define SYSTICK_CTRL     (*(volatile unsigned long*) 0xE000E010)   /* SysTick Control and Status Register */
#define SYSTICK_LOAD     (*(volatile unsigned long*) 0xE000E014)   /* SysTick Reload Value Register       */
#define SYSTICK_VAL      (*(volatile unsigned long*) 0xE000E018)   /* SysTick Current Value Register      */
#define SYSTICK_CALIB    (*(volatile unsigned long*) 0xE000E01C)   /* SysTick Calibration Register        */





/* Reset und Clock Control.. wo: RefManual ab Seite 75 */
struct T_RCC
{ volatile unsigned long CR;
  volatile unsigned long CFGR;
  volatile unsigned long CIR;
  volatile unsigned long APB2RSTR;
  volatile unsigned long APB1RSTR;
  volatile unsigned long AHBENR;
  volatile unsigned long APB2ENR;
  volatile unsigned long APB1ENR;
  volatile unsigned long BDCR;
  volatile unsigned long CSR;
};

#define RCC   ((struct T_RCC*) 0x40021000)

/* embedded Flash..  wo: Flash programming Manual */
struct T_FLASH
{ volatile unsigned long ACR;
  volatile unsigned long KEYR;
  volatile unsigned long OPTKEYR;
  volatile unsigned long SR;
  volatile unsigned long CR;
  volatile unsigned long AR;
  volatile unsigned long RESERVED;
  volatile unsigned long OBR;
  volatile unsigned long WRPR;
};

#define FLASH   ((struct T_FLASH*) 0x40022000)

/* Power-Control.. wo: RefManual ab Seite 54 */
struct T_PWR
{ volatile unsigned long CR;
  volatile unsigned long CSR;
};

#define PWR   ((struct T_PWR*) 0x40007000)

/* USART's .. wo: RefManual ab Seite 666 */
struct T_USART
{ volatile unsigned long SR;   /* Status*/
  volatile unsigned long DR;   /* Data Register Senden+Empfangen */
  volatile unsigned long BRR;  /* Baudrate */
  volatile unsigned long CR1;  /* Steuerregister 1 */
  volatile unsigned long CR2;  /* Steuerregister 2 */
  volatile unsigned long CR3;  /* Steuerregister 3 */
  volatile unsigned long GTPR; /* Guardtime + Prescaler */
};
/* USART1 hngt am APB2 */
#define USART1   ((struct T_USART*) 0x40013800)
/* alle anderen U(S)ART's hngen am APB1 */
#define USART2   ((struct T_USART*) 0x40004400)
#define USART3   ((struct T_USART*) 0x40004800)
#define UART4    ((struct T_USART*) 0x40004C00)
#define UART5    ((struct T_USART*) 0x40005000)


/* Ports PA..PG ..wo: RefManual ab Seite 139 */
struct T_GPIO
{ volatile unsigned long CRL;   /* Config Bits 0..7  */
  volatile unsigned long CRH;   /* Config Bits 8..15 */
  volatile unsigned long IDR;   /* Port-Inputs (r/o) */
  volatile unsigned long ODR;   /* Port-Outputs (r/w) */
  volatile unsigned long BSRR;  /* Bit-SetReset:
                                   Bits 0..15->set ODR Bits 0..15,
				   Bits 16..31->reset ODR Bits 0..15 */
  volatile unsigned long BRR;   /* Bit-Reset: Bits 0..15->reset ODR Bits 0..15 */
  volatile unsigned long LCKR;  /* Port lock: freeze CRL, CRH  */
};

#define GPIOA   ((struct T_GPIO*) 0x40010800)
#define GPIOB   ((struct T_GPIO*) 0x40010C00)
#define GPIOC   ((struct T_GPIO*) 0x40011000)
#define GPIOD   ((struct T_GPIO*) 0x40011400)
#define GPIOE   ((struct T_GPIO*) 0x40011800)
#define GPIOF   ((struct T_GPIO*) 0x40011C00)
#define GPIOG   ((struct T_GPIO*) 0x40012000)

/* Hilfsbezeichnungen zum Port-Konfigurieren */
/* Beispiel: #define wert_fuer_pa0  (mode?+conf?)  */
#define input   0     /* Pin ist Input */
#define out10   1     /* Pin ist Output, max. 10 MHz schnell */
#define out2    2     /* Pin ist Output, max.  2 MHz schnell */
#define out50   3     /* Pin ist Output, max. 50 MHz schnell */

#define conf0   0     /* in:analog, out:gpio */
#define conf1   4     /* in:hi-z, out:open drain */
#define conf2   8     /* in:pullup/dn, out:alternate function */
#define conf3  12     /* in:verboten, out:alternate fkt mit open drain */

/* aufgedrselte Port-Konfiguationen: */
/* CNF=0 */
#define in_analog        0
#define out10_gpo        1
#define out2_gpo         2
#define out50_gpo        3
/* CNF=1 */
#define in_gpi           4
#define out10_od         5
#define out2_od          6
#define out50_od         7
/* CNF=2 */
#define in_pupd          8
#define out10_altfu      9
#define out2_altfu      10
#define out50_altfu     11
/* CNF=3 */
#define verboten        12
#define out10_altfu_od  13
#define out2_altfu_od   14
#define out50_altfu_od  15


/* Alternate Function I/O, wo: RefManual ab Seite 154 */
/* zum Plazieren von Peripherie auf verschiedene Pins */
struct T_AFIO
{ volatile unsigned long EVCR;
  volatile unsigned long MAPR;
  volatile unsigned long EXTICR[4];
};

#define AFIO   ((struct T_AFIO*) 0x40010000)

/* Externer Bus */

#define FSMC_BCR1  (*(volatile unsigned long*) 0xA0000000)
#define FSMC_BTR1  (*(volatile unsigned long*) 0xA0000004)

#define FSMC_BCR2  (*(volatile unsigned long*) 0xA0000008)
#define FSMC_BTR2  (*(volatile unsigned long*) 0xA000000C)

#define FSMC_BCR3  (*(volatile unsigned long*) 0xA0000010)
#define FSMC_BTR3  (*(volatile unsigned long*) 0xA0000014)

#define FSMC_BCR4  (*(volatile unsigned long*) 0xA0000018)
#define FSMC_BTR4  (*(volatile unsigned long*) 0xA000001C)

#define FSMC_BWTR1 (*(volatile unsigned long*) 0xA0000104)
#define FSMC_BWTR2 (*(volatile unsigned long*) 0xA000010C)
#define FSMC_BWTR3 (*(volatile unsigned long*) 0xA0000114)
#define FSMC_BWTR4 (*(volatile unsigned long*) 0xA000011C)

#define FSMC_PCR2  (*(volatile unsigned long*) 0xA0000060)
#define FSMC_PCR3  (*(volatile unsigned long*) 0xA0000080)
#define FSMC_PCR4  (*(volatile unsigned long*) 0xA00000A0)

#define FSMC_SR2   (*(volatile unsigned long*) 0xA0000064)
#define FSMC_SR3   (*(volatile unsigned long*) 0xA0000084)
#define FSMC_SR4   (*(volatile unsigned long*) 0xA00000A4)

#define FSMC_PMEM2 (*(volatile unsigned long*) 0xA0000068)
#define FSMC_PMEM3 (*(volatile unsigned long*) 0xA0000088)
#define FSMC_PMEM4 (*(volatile unsigned long*) 0xA00000A8)

#define FSMC_PATT2 (*(volatile unsigned long*) 0xA000006C)
#define FSMC_PATT3 (*(volatile unsigned long*) 0xA000008C)
#define FSMC_PATT4 (*(volatile unsigned long*) 0xA00000AC)

#define FSMC_PIO4  (*(volatile unsigned long*) 0xA00000B0)

#define FSMC_ECCR2 (*(volatile unsigned long*) 0xA0000054)
#define FSMC_ECCR3 (*(volatile unsigned long*) 0xA0000074)


/* SDIO Speicherkarten-Interface */

#define SDIO_POWER   (*(volatile unsigned long*) 0x40018000)
#define SDIO_CLKCR   (*(volatile unsigned long*) 0x40018004)
#define SDIO_ARG     (*(volatile unsigned long*) 0x40018008)
#define SDIO_CMD     (*(volatile unsigned long*) 0x4001800C)
#define SDIO_RESPCMD (*(volatile unsigned long*) 0x40018010)
#define SDIO_RESP1   (*(volatile unsigned long*) 0x40018014)
#define SDIO_RESP2   (*(volatile unsigned long*) 0x40018018)
#define SDIO_RESP3   (*(volatile unsigned long*) 0x4001801C)
#define SDIO_RESP4   (*(volatile unsigned long*) 0x40018020)
#define SDIO_DTIMER  (*(volatile unsigned long*) 0x40018024)
#define SDIO_DLEN    (*(volatile unsigned long*) 0x40018028)
#define SDIO_DCTRL   (*(volatile unsigned long*) 0x4001802C)
#define SDIO_DCOUNT  (*(volatile unsigned long*) 0x40018030)
#define SDIO_STA     (*(volatile unsigned long*) 0x40018034)
#define SDIO_ICR     (*(volatile unsigned long*) 0x40018038)
#define SDIO_MASK    (*(volatile unsigned long*) 0x4001803C)
#define SDIO_FIFOCNT (*(volatile unsigned long*) 0x40018048)
#define SDIO_FIFO    (*(volatile unsigned long*) 0x40018080)


/* DMA */

#define DMA1_ISR     (*(volatile unsigned long*) 0x40020000)
#define DMA1_IFCR    (*(volatile unsigned long*) 0x40020004)

#define DMA1_CCR1    (*(volatile unsigned long*) 0x40020008)
#define DMA1_CNDTR1  (*(volatile unsigned long*) 0x4002000C)
#define DMA1_CPAR1   (*(volatile unsigned long*) 0x40020010)
#define DMA1_CMAR1   (*(volatile unsigned long*) 0x40020014)

#define DMA1_CCR2    (*(volatile unsigned long*) 0x4002001C)
#define DMA1_CNDTR2  (*(volatile unsigned long*) 0x40020020)
#define DMA1_CPAR2   (*(volatile unsigned long*) 0x40020024)
#define DMA1_CMAR2   (*(volatile unsigned long*) 0x40020028)

#define DMA1_CCR3    (*(volatile unsigned long*) 0x40020030)
#define DMA1_CNDTR3  (*(volatile unsigned long*) 0x40020034)
#define DMA1_CPAR3   (*(volatile unsigned long*) 0x40020038)
#define DMA1_CMAR3   (*(volatile unsigned long*) 0x4002003C)

#define DMA1_CCR4    (*(volatile unsigned long*) 0x40020044)
#define DMA1_CNDTR4  (*(volatile unsigned long*) 0x40020048)
#define DMA1_CPAR4   (*(volatile unsigned long*) 0x4002004C)
#define DMA1_CMAR4   (*(volatile unsigned long*) 0x40020050)

#define DMA1_CCR5    (*(volatile unsigned long*) 0x40020058)
#define DMA1_CNDTR5  (*(volatile unsigned long*) 0x4002005C)
#define DMA1_CPAR5   (*(volatile unsigned long*) 0x40020060)
#define DMA1_CMAR5   (*(volatile unsigned long*) 0x40020064)

#define DMA1_CCR6    (*(volatile unsigned long*) 0x4002006C)
#define DMA1_CNDTR6  (*(volatile unsigned long*) 0x40020070)
#define DMA1_CPAR6   (*(volatile unsigned long*) 0x40020074)
#define DMA1_CMAR6   (*(volatile unsigned long*) 0x40020078)

#define DMA1_CCR7    (*(volatile unsigned long*) 0x40020080)
#define DMA1_CNDTR7  (*(volatile unsigned long*) 0x40020084)
#define DMA1_CPAR7   (*(volatile unsigned long*) 0x40020088)
#define DMA1_CMAR7   (*(volatile unsigned long*) 0x4002008C)


#define DMA2_ISR     (*(volatile unsigned long*) 0x40020400)
#define DMA2_IFCR    (*(volatile unsigned long*) 0x40020404)

#define DMA2_CCR1    (*(volatile unsigned long*) 0x40020408)
#define DMA2_CNDTR1  (*(volatile unsigned long*) 0x4002040C)
#define DMA2_CPAR1   (*(volatile unsigned long*) 0x40020410)
#define DMA2_CMAR1   (*(volatile unsigned long*) 0x40020414)

#define DMA2_CCR2    (*(volatile unsigned long*) 0x4002041C)
#define DMA2_CNDTR2  (*(volatile unsigned long*) 0x40020420)
#define DMA2_CPAR2   (*(volatile unsigned long*) 0x40020424)
#define DMA2_CMAR2   (*(volatile unsigned long*) 0x40020428)

#define DMA2_CCR3    (*(volatile unsigned long*) 0x40020430)
#define DMA2_CNDTR3  (*(volatile unsigned long*) 0x40020434)
#define DMA2_CPAR3   (*(volatile unsigned long*) 0x40020438)
#define DMA2_CMAR3   (*(volatile unsigned long*) 0x4002043C)

#define DMA2_CCR4    (*(volatile unsigned long*) 0x40020444)
#define DMA2_CNDTR4  (*(volatile unsigned long*) 0x40020448)
#define DMA2_CPAR4   (*(volatile unsigned long*) 0x4002044C)
#define DMA2_CMAR4   (*(volatile unsigned long*) 0x40020450)

#define DMA2_CCR5    (*(volatile unsigned long*) 0x40020458)
#define DMA2_CNDTR5  (*(volatile unsigned long*) 0x4002045C)
#define DMA2_CPAR5   (*(volatile unsigned long*) 0x40020460)
#define DMA2_CMAR5   (*(volatile unsigned long*) 0x40020464)


/* Interrupt-Nummern */

/******  Cortex-M3 Processor Exceptions Numbers *****/
#define  NonMaskableInt_IRQn         -14     /*  2 Non Maskable Interrupt */
#define  MemoryManagement_IRQn       -12     /*  4 Cortex-M3 Memory Management Interrupt  */
#define  BusFault_IRQn               -11     /*  5 Cortex-M3 Bus Fault Interrupt     */
#define  UsageFault_IRQn             -10     /*  6 Cortex-M3 Usage Fault Interrupt   */
#define  SVCall_IRQn                  -5     /* 11 Cortex-M3 SV Call Interrupt       */
#define  DebugMonitor_IRQn            -4     /* 12 Cortex-M3 Debug Monitor Interrupt */
#define  PendSV_IRQn                  -2     /* 14 Cortex-M3 Pend SV Interrupt       */
#define  SysTick_IRQn                 -1     /* 15 Cortex-M3 System Tick Interrupt   */

/******  STM32 specific Interrupt Numbers *********************************************************/
#define  WWDG_IRQn                    0      /* Window WatchDog Interrupt                            */
#define  PVD_IRQn                     1      /* PVD through EXTI Line detection Interrupt            */
#define  TAMPER_IRQn                  2      /* Tamper Interrupt                                     */
#define  RTC_IRQn                     3      /* RTC global Interrupt                                 */
#define  FLASH_IRQn                   4      /* FLASH global Interrupt                               */
#define  RCC_IRQn                     5      /* RCC global Interrupt                                 */
#define  EXTI0_IRQn                   6      /* EXTI Line0 Interrupt                                 */
#define  EXTI1_IRQn                   7      /* EXTI Line1 Interrupt                                 */
#define  EXTI2_IRQn                   8      /* EXTI Line2 Interrupt                                 */
#define  EXTI3_IRQn                   9      /* EXTI Line3 Interrupt                                 */
#define  EXTI4_IRQn                   10     /* EXTI Line4 Interrupt                                 */
#define  DMA1_Channel1_IRQn           11     /* DMA1 Channel 1 global Interrupt                      */
#define  DMA1_Channel2_IRQn           12     /* DMA1 Channel 2 global Interrupt                      */
#define  DMA1_Channel3_IRQn           13     /* DMA1 Channel 3 global Interrupt                      */
#define  DMA1_Channel4_IRQn           14     /* DMA1 Channel 4 global Interrupt                      */
#define  DMA1_Channel5_IRQn           15     /* DMA1 Channel 5 global Interrupt                      */
#define  DMA1_Channel6_IRQn           16     /* DMA1 Channel 6 global Interrupt                      */
#define  DMA1_Channel7_IRQn           17     /* DMA1 Channel 7 global Interrupt                      */
#define  ADC1_2_IRQn                  18     /* ADC1 et ADC2 global Interrupt                        */
#define  USB_HP_CAN1_TX_IRQn          19     /* USB High Priority or CAN1 TX Interrupts              */
#define  USB_LP_CAN1_RX0_IRQn         20     /* USB Low Priority or CAN1 RX0 Interrupts              */
#define  CAN1_RX1_IRQn                21     /* CAN1 RX1 Interrupt                                   */
#define  CAN1_SCE_IRQn                22     /* CAN1 SCE Interrupt                                   */
#define  EXTI9_5_IRQn                 23     /* External Line[9:5] Interrupts                        */
#define  TIM1_BRK_IRQn                24     /* TIM1 Break Interrupt                                 */
#define  TIM1_UP_IRQn                 25     /* TIM1 Update Interrupt                                */
#define  TIM1_TRG_COM_IRQn            26     /* TIM1 Trigger and Commutation Interrupt               */
#define  TIM1_CC_IRQn                 27     /* TIM1 Capture Compare Interrupt                       */
#define  TIM2_IRQn                    28     /* TIM2 global Interrupt                                */
#define  TIM3_IRQn                    29     /* TIM3 global Interrupt                                */
#define  TIM4_IRQn                    30     /* TIM4 global Interrupt                                */
#define  I2C1_EV_IRQn                 31     /* I2C1 Event Interrupt                                 */
#define  I2C1_ER_IRQn                 32     /* I2C1 Error Interrupt                                 */
#define  I2C2_EV_IRQn                 33     /* I2C2 Event Interrupt                                 */
#define  I2C2_ER_IRQn                 34     /* I2C2 Error Interrupt                                 */
#define  SPI1_IRQn                    35     /* SPI1 global Interrupt                                */
#define  SPI2_IRQn                    36     /* SPI2 global Interrupt                                */
#define  USART1_IRQn                  37     /* USART1 global Interrupt                              */
#define  USART2_IRQn                  38     /* USART2 global Interrupt                              */
#define  USART3_IRQn                  39     /* USART3 global Interrupt                              */
#define  EXTI15_10_IRQn               40     /* External Line[15:10] Interrupts                      */
#define  RTCAlarm_IRQn                41     /* RTC Alarm through EXTI Line Interrupt                */
#define  USBWakeUp_IRQn               42     /* USB WakeUp from suspend through EXTI Line Interrupt  */
#define  TIM8_BRK_IRQn                43     /* TIM8 Break Interrupt                                 */
#define  TIM8_UP_IRQn                 44     /* TIM8 Update Interrupt                                */
#define  TIM8_TRG_COM_IRQn            45     /* TIM8 Trigger and Commutation Interrupt               */
#define  TIM8_CC_IRQn                 46     /* TIM8 Capture Compare Interrupt                       */
#define  ADC3_IRQn                    47     /* ADC3 global Interrupt                                */
#define  FSMC_IRQn                    48     /* FSMC global Interrupt                                */
#define  SDIO_IRQn                    49     /* SDIO global Interrupt                                */
#define  TIM5_IRQn                    50     /* TIM5 global Interrupt                                */
#define  SPI3_IRQn                    51     /* SPI3 global Interrupt                                */
#define  UART4_IRQn                   52     /* UART4 global Interrupt                               */
#define  UART5_IRQn                   53     /* UART5 global Interrupt                               */
#define  TIM6_IRQn                    54     /* TIM6 global Interrupt                                */
#define  TIM7_IRQn                    55     /* TIM7 global Interrupt                                */
#define  DMA2_Channel1_IRQn           56     /* DMA2 Channel 1 global Interrupt                      */
#define  DMA2_Channel2_IRQn           57     /* DMA2 Channel 2 global Interrupt                      */
#define  DMA2_Channel3_IRQn           58     /* DMA2 Channel 3 global Interrupt                      */
#define  DMA2_Channel4_5_IRQn         59     /* DMA2 Channel 4 and Channel 5 global Interrupt        */



#endif

